@@ -151,6 +151,19 @@ class Offset(NamedTuple):
         distance: float = ((x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1)) ** 0.5
         return distance
 
+    def clamp(self, width: int, height: int) -> Offset:
+        """Clamp the offset to fit within a rectangle of width x height.
+
+        Args:
+            width: Width to clamp.
+            height: Height to clamp.
+
+        Returns:
+            A new offset.
+        """
+        x, y = self
+        return Offset(clamp(x, 0, width - 1), clamp(y, 0, height - 1))
+
 
 class Size(NamedTuple):
     """The dimensions (width and height) of a rectangular region.
@@ -194,6 +207,28 @@ class Size(NamedTuple):
         """A range object that covers values between 0 and `height`."""
         return range(self.height)
 
+    def with_width(self, width: int) -> Size:
+        """Get a new Size with just the width changed.
+
+        Args:
+            width: New width.
+
+        Returns:
+            New Size instance.
+        """
+        return Size(width, self.height)
+
+    def with_height(self, height: int) -> Size:
+        """Get a new Size with just the height changed.
+
+        Args:
+            width: New height.
+
+        Returns:
+            New Size instance.
+        """
+        return Size(self.width, height)
+
     def __add__(self, other: object) -> Size:
         if isinstance(other, tuple):
             width, height = self
@@ -246,6 +281,17 @@ class Size(NamedTuple):
         width, height = self
         return width > x >= 0 and height > y >= 0
 
+    def clamp_offset(self, offset: Offset) -> Offset:
+        """Clamp an offset to fit within the width x height.
+
+        Args:
+            offset: An offset.
+
+        Returns:
+            A new offset that will fit inside the dimensions defined in the Size.
+        """
+        return offset.clamp(self.width, self.height)
+
 
 class Region(NamedTuple):
     """Defines a rectangular region.
@@ -427,7 +473,7 @@ class Region(NamedTuple):
 
     @property
     def area(self) -> int:
-        """The are under the region."""
+        """The area under the region."""
         return self.width * self.height
 
     @property
@@ -439,6 +485,18 @@ class Region(NamedTuple):
         """
         return Offset(*self[:2])
 
+    @property
+    def center(self) -> tuple[float, float]:
+        """The center of the region.
+
+        Note, that this does *not* return an `Offset`, because the center may not be an integer coordinate.
+
+        Returns:
+            Tuple of floats.
+        """
+        x, y, width, height = self
+        return (x + width / 2.0, y + height / 2.0)
+
     @property
     def bottom_left(self) -> Offset:
         """Bottom left offset of the region.
@@ -839,7 +897,7 @@ class Region(NamedTuple):
 
     @lru_cache(maxsize=1024)
     def split_horizontal(self, cut: int) -> tuple[Region, Region]:
-        """Split a region in to two, from a given x offset.
+        """Split a region in to two, from a given y offset.
 
         ```
                     ┌─────────┐
@@ -852,8 +910,8 @@ class Region(NamedTuple):
         ```
 
         Args:
-            cut: An offset from self.x where the cut should be made. May be negative,
-                for the offset to start from the right edge.
+            cut: An offset from self.y where the cut should be made. May be negative,
+                for the offset to start from the lower edge.
 
         Returns:
             Two regions, which add up to the original (self).
@@ -867,11 +925,102 @@ class Region(NamedTuple):
             Region(x, y + cut, width, height - cut),
         )
 
+    def translate_inside(
+        self, container: Region, x_axis: bool = True, y_axis: bool = True
+    ) -> Region:
+        """Translate this region, so it fits within a container.
+
+        This will ensure that there is as little overlap as possible.
+        The top left of the returned region is guaranteed to be within the container.
+
+        ```
+        ┌──────────────────┐         ┌──────────────────┐
+        │    container     │         │    container     │
+        │                  │         │    ┌─────────────┤
+        │                  │   ──▶   │    │    return   │
+        │       ┌──────────┴──┐      │    │             │
+        │       │    self     │      │    │             │
+        └───────┤             │      └────┴─────────────┘
+                │             │
+                └─────────────┘
+        ```
+
+
+        Args:
+            container: A container region.
+            x_axis: Allow translation of X axis.
+            y_axis: Allow translation of Y axis.
+
+        Returns:
+            A new region with same dimensions that fits with inside container.
+        """
+        x1, y1, width1, height1 = container
+        x2, y2, width2, height2 = self
+        return Region(
+            max(min(x2, x1 + width1 - width2), x1) if x_axis else x2,
+            max(min(y2, y1 + height1 - height2), y1) if y_axis else y2,
+            width2,
+            height2,
+        )
+
+    def inflect(
+        self, x_axis: int = +1, y_axis: int = +1, margin: Spacing | None = None
+    ) -> Region:
+        """Inflect a region around one or both axis.
+
+        The `x_axis` and `y_axis` parameters define which direction to move the region.
+        A positive value will move the region right or down, a negative value will move
+        the region left or up. A value of `0` will leave that axis unmodified.
+
+        ```
+        ╔══════════╗    │
+        ║          ║
+        ║   Self   ║    │
+        ║          ║
+        ╚══════════╝    │
+
+        ─ ─ ─ ─ ─ ─ ─ ─ ┼ ─ ─ ─ ─ ─ ─ ─ ─
+
+                        │    ┌──────────┐
+                             │          │
+                        │    │  Result  │
+                             │          │
+                        │    └──────────┘
+        ```
+
+        Args:
+            x_axis: +1 to inflect in the positive direction, -1 to inflect in the negative direction.
+            y_axis: +1 to inflect in the positive direction, -1 to inflect in the negative direction.
+            margin: Additional margin.
+
+        Returns:
+            A new region.
+        """
+        inflect_margin = NULL_SPACING if margin is None else margin
+        x, y, width, height = self
+        if x_axis:
+            x += (width + inflect_margin.width) * x_axis
+        if y_axis:
+            y += (height + inflect_margin.height) * y_axis
+        return Region(x, y, width, height)
+
 
 class Spacing(NamedTuple):
-    """The spacing around a renderable, such as padding and border
+    """The spacing around a renderable, such as padding and border.
 
-    Spacing is defined by four integers for the space at the top, right, bottom, and left of a region,
+    Spacing is defined by four integers for the space at the top, right, bottom, and left of a region.
+
+    ```
+    ┌ ─ ─ ─ ─ ─ ─ ─▲─ ─ ─ ─ ─ ─ ─ ─ ┐
+                   │ top
+    │        ┏━━━━━▼━━━━━━┓         │
+     ◀──────▶┃            ┃◀───────▶
+    │  left  ┃            ┃ right   │
+             ┃            ┃
+    │        ┗━━━━━▲━━━━━━┛         │
+                   │ bottom
+    └ ─ ─ ─ ─ ─ ─ ─▼─ ─ ─ ─ ─ ─ ─ ─ ┘
+    ```
 
     Example:
         ```python
@@ -890,7 +1039,7 @@ class Spacing(NamedTuple):
     top: int = 0
     """Space from the top of a region."""
     right: int = 0
-    """Space from the left of a region."""
+    """Space from the right of a region."""
     bottom: int = 0
     """Space from the bottom of a region."""
     left: int = 0
@@ -974,7 +1123,7 @@ class Spacing(NamedTuple):
         and no horizontal spacing.
 
         Args:
-            amount: The magnitude of spacing to apply to vertical edges
+            amount: The magnitude of spacing to apply to vertical edges.
 
         Returns:
             `Spacing(amount, 0, amount, 0)`
@@ -987,7 +1136,7 @@ class Spacing(NamedTuple):
         and no vertical spacing.
 
         Args:
-            amount: The magnitude of spacing to apply to horizontal edges
+            amount: The magnitude of spacing to apply to horizontal edges.
 
         Returns:
             `Spacing(0, amount, 0, amount)`
@@ -999,7 +1148,7 @@ class Spacing(NamedTuple):
         """Construct a Spacing with a given amount of spacing on all edges.
 
         Args:
-            amount: The magnitude of spacing to apply to all edges
+            amount: The magnitude of spacing to apply to all edges.
 
         Returns:
             `Spacing(amount, amount, amount, amount)`
@@ -1031,7 +1180,7 @@ class Spacing(NamedTuple):
             other: Spacing object.
 
         Returns:
-            New spacing were the values are maximum of the two values.
+            New spacing where the values are maximum of the two values.
         """
         top, right, bottom, left = self
         other_top, other_right, other_bottom, other_left = other
@@ -1045,3 +1194,12 @@ class Spacing(NamedTuple):
 
 NULL_OFFSET: Final = Offset(0, 0)
 """An [offset][textual.geometry.Offset] constant for (0, 0)."""
+
+NULL_REGION: Final = Region(0, 0, 0, 0)
+"""A [Region][textual.geometry.Region] constant for a null region (at the origin, with both width and height set to zero)."""
+
+NULL_SIZE: Final = Size(0, 0)
+"""A [Size][textual.geometry.Size] constant for a null size (with zero area)."""
+
+NULL_SPACING: Final = Spacing(0, 0, 0, 0)
+"""A [Spacing][textual.geometry.Spacing] constant for no space."""
