@@ -5,6 +5,7 @@ from dataclasses import dataclass
 from typing import ClassVar
 
 import rich.repr
+from rich.console import RenderableType
 from rich.style import Style
 from rich.text import Text, TextType
 
@@ -41,7 +42,7 @@ class Underline(Widget):
     """
     | Class | Description |
     | :- | :- |
-    | `underline-bar` | Style of the bar (may be used to change the color). |
+    | `underline--bar` | Style of the bar (may be used to change the color). |
     """
 
     highlight_start = reactive(0)
@@ -143,6 +144,9 @@ class Tab(Static):
     class Enabled(TabMessage):
         """A tab was enabled."""
 
+    class Relabelled(TabMessage):
+        """A tab was relabelled."""
+
     def __init__(
         self,
         label: TextType,
@@ -159,9 +163,24 @@ class Tab(Static):
             classes: Space separated list of class names.
             disabled: Whether the tab is disabled or not.
         """
-        self.label = Text.from_markup(label) if isinstance(label, str) else label
         super().__init__(id=id, classes=classes, disabled=disabled)
-        self.update(label)
+        self._label: Text
+        # Setter takes Text or str
+        self.label = label  # type: ignore[assignment]
+
+    @property
+    def label(self) -> Text:
+        """The label for the tab."""
+        return self._label
+
+    @label.setter
+    def label(self, label: TextType) -> None:
+        self._label = Text.from_markup(label) if isinstance(label, str) else label
+        self.update(self._label)
+
+    def update(self, renderable: RenderableType = "") -> None:
+        self.post_message(self.Relabelled(self))
+        return super().update(renderable)
 
     @property
     def label_text(self) -> str:
@@ -267,7 +286,8 @@ class Tabs(Widget, can_focus=True):
     class Cleared(Message):
         """Sent when there are no active tabs.
 
-        This can occur when Tabs are cleared, or if all tabs are hidden.
+        This can occur when Tabs are cleared, if all tabs are hidden, or if the
+        currently active tab is unset.
         """
 
         def __init__(self, tabs: Tabs) -> None:
@@ -478,7 +498,7 @@ class Tabs(Widget, can_focus=True):
         underline.highlight_end = 0
         self.call_after_refresh(self.post_message, self.Cleared(self))
         self.active = ""
-        return AwaitComplete(self.query("#tabs-list > Tab").remove()())
+        return AwaitComplete(self.query("#tabs-list > Tab").remove())
 
     def remove_tab(self, tab_or_id: Tab | str | None) -> AwaitComplete:
         """Remove a tab.
@@ -490,7 +510,7 @@ class Tabs(Widget, can_focus=True):
             An optionally awaitable object that waits for the tab to be removed.
         """
         if not tab_or_id:
-            return AwaitComplete(self.app._remove_nodes([], None)())
+            return AwaitComplete(self.app._remove_nodes([], None))
 
         if isinstance(tab_or_id, Tab):
             remove_tab = tab_or_id
@@ -498,7 +518,7 @@ class Tabs(Widget, can_focus=True):
             try:
                 remove_tab = self.query_one(f"#tabs-list > #{tab_or_id}", Tab)
             except NoMatches:
-                return AwaitComplete(self.app._remove_nodes([], None)())
+                return AwaitComplete(self.app._remove_nodes([], None))
 
         removing_active_tab = remove_tab.has_class("-active")
         next_tab = self._next_active
@@ -509,10 +529,10 @@ class Tabs(Widget, can_focus=True):
         async def do_remove() -> None:
             """Perform the remove after refresh so the underline bar gets new positions."""
             await remove_await
-            if next_tab is None:
+            if next_tab is None or (removing_active_tab and next_tab.id is None):
                 self.active = ""
             elif removing_active_tab:
-                self.active = next_tab.id
+                self.active = next_tab.id or ""
                 next_tab.add_class("-active")
 
             highlight_updated.set()
@@ -557,14 +577,15 @@ class Tabs(Widget, can_focus=True):
 
     def watch_active(self, previously_active: str, active: str) -> None:
         """Handle a change to the active tab."""
+        self.query("#tabs-list > Tab.-active").remove_class("-active")
         if active:
             try:
                 active_tab = self.query_one(f"#tabs-list > #{active}", Tab)
             except NoMatches:
                 return
-            self.query("#tabs-list > Tab.-active").remove_class("-active")
             active_tab.add_class("-active")
             self._highlight_active(animate=previously_active != "")
+            self._scroll_active_tab()
             self.post_message(self.TabActivated(self, active_tab))
         else:
             underline = self.query_one(Underline)
@@ -572,7 +593,10 @@ class Tabs(Widget, can_focus=True):
             underline.highlight_end = 0
             self.post_message(self.Cleared(self))
 
-    def _highlight_active(self, animate: bool = True) -> None:
+    def _highlight_active(
+        self,
+        animate: bool = True,
+    ) -> None:
         """Move the underline bar to under the active tab.
 
         Args:
@@ -589,9 +613,34 @@ class Tabs(Widget, can_focus=True):
             underline.show_highlight = True
             tab_region = active_tab.virtual_region.shrink(active_tab.styles.gutter)
             start, end = tab_region.column_span
-            if animate:
-                underline.animate("highlight_start", start, duration=0.3)
-                underline.animate("highlight_end", end, duration=0.3)
+            # This is a basic animation, so we only disable it if we want no animations.
+            if animate and self.app.animation_level != "none":
+
+                def animate_underline() -> None:
+                    """Animate the underline."""
+                    try:
+                        active_tab = self.query_one(f"#tabs-list > Tab.-active")
+                    except NoMatches:
+                        pass
+                    else:
+                        tab_region = active_tab.virtual_region.shrink(
+                            active_tab.styles.gutter
+                        )
+                        start, end = tab_region.column_span
+                        underline.animate(
+                            "highlight_start",
+                            start,
+                            duration=0.3,
+                            level="basic",
+                        )
+                        underline.animate(
+                            "highlight_end",
+                            end,
+                            duration=0.3,
+                            level="basic",
+                        )
+
+                self.set_timer(0.02, lambda: self.call_after_refresh(animate_underline))
             else:
                 underline.highlight_start = start
                 underline.highlight_end = end
@@ -611,7 +660,6 @@ class Tabs(Widget, can_focus=True):
         self.query("#tabs-list Tab.-active").remove_class("-active")
         tab.add_class("-active")
         self.active = tab.id or ""
-        self.query_one("#tabs-scroll").scroll_to_center(tab, force=True)
 
     def _on_underline_clicked(self, event: Underline.Clicked) -> None:
         """The underline was clicked.
@@ -653,21 +701,24 @@ class Tabs(Widget, can_focus=True):
         self._move_tab(-1)
 
     def _move_tab(self, direction: int) -> None:
-        """Activate the next tab.
+        """Activate the next enabled tab in the given direction.
+
+        Tab selection wraps around. If no tab is currently active, the "next"
+        tab is set to be the first and the "previous" tab is the last one.
 
         Args:
             direction: +1 for the next tab, -1 for the previous.
         """
         active_tab = self.active_tab
-        if active_tab is None:
-            return
         tabs = self._potentially_active_tabs
         if not tabs:
             return
+        if not active_tab:
+            self.active = tabs[0 if direction == 1 else -1].id or ""
+            return
         tab_count = len(tabs)
         new_tab_index = (tabs.index(active_tab) + direction) % tab_count
         self.active = tabs[new_tab_index].id or ""
-        self._scroll_active_tab()
 
     def _on_tab_disabled(self, event: Tab.Disabled) -> None:
         """Re-post the disabled message."""
@@ -679,6 +730,11 @@ class Tabs(Widget, can_focus=True):
         event.stop()
         self.post_message(self.TabEnabled(self, event.tab))
 
+    def _on_tab_relabelled(self, event: Tab.Relabelled) -> None:
+        """Redraw the highlight when tab is relabelled."""
+        event.stop()
+        self._highlight_active()
+
     def disable(self, tab_id: str) -> Tab:
         """Disable the indicated tab.
 
@@ -747,7 +803,7 @@ class Tabs(Widget, can_focus=True):
             next_tab = self._next_active
             self.active = next_tab.id or "" if next_tab else ""
         tab_to_hide.add_class("-hidden")
-        self.post_message(self.TabHidden(self, tab_to_hide))
+        self.post_message(self.TabHidden(self, tab_to_hide).set_sender(self))
         self.call_after_refresh(self._highlight_active)
         return tab_to_hide
 
@@ -770,7 +826,7 @@ class Tabs(Widget, can_focus=True):
             raise self.TabError(f"There is no tab with ID {tab_id!r} to show.")
 
         tab_to_show.remove_class("-hidden")
-        self.post_message(self.TabShown(self, tab_to_show))
+        self.post_message(self.TabShown(self, tab_to_show).set_sender(self))
         if not self.active:
             self._activate_tab(tab_to_show)
         self.call_after_refresh(self._highlight_active)
