@@ -1,17 +1,17 @@
-import sys
 from pathlib import Path
 
 import pytest
 
 from tests.snapshot_tests.language_snippets import SNIPPETS
 from textual.widgets.text_area import Selection, BUILTIN_LANGUAGES
-from textual.widgets import TextArea, Input, Button
+from textual.widgets import RichLog, TextArea, Input, Button
 from textual.widgets.text_area import TextAreaTheme
 
 # These paths should be relative to THIS directory.
 WIDGET_EXAMPLES_DIR = Path("../../docs/examples/widgets")
 LAYOUT_EXAMPLES_DIR = Path("../../docs/examples/guide/layout")
 STYLES_EXAMPLES_DIR = Path("../../docs/examples/styles")
+EXAMPLES_DIR = Path("../../examples")
 SNAPSHOT_APPS_DIR = Path("./snapshot_apps")
 
 
@@ -103,10 +103,10 @@ def test_input_validation(snap_compare):
 
 def test_input_suggestions(snap_compare):
     async def run_before(pilot):
-        pilot.app.query_one(Input).cursor_blink = False
+        pilot.app.query(Input).first().cursor_blink = False
 
     assert snap_compare(
-        SNAPSHOT_APPS_DIR / "input_suggestions.py", press=[], run_before=run_before
+        SNAPSHOT_APPS_DIR / "input_suggestions.py", press=["b"], run_before=run_before
     )
 
 
@@ -230,6 +230,22 @@ def test_markdown_viewer_example(snap_compare):
     assert snap_compare(WIDGET_EXAMPLES_DIR / "markdown_viewer.py")
 
 
+def test_markdown_theme_switching(snap_compare):
+    assert snap_compare(SNAPSHOT_APPS_DIR / "markdown_theme_switcher.py", press=["t"])
+
+
+def test_markdown_dark_theme_override(snap_compare):
+    assert snap_compare(
+        SNAPSHOT_APPS_DIR / "markdown_theme_switcher.py", press=["d", "wait:100"]
+    )
+
+
+def test_markdown_light_theme_override(snap_compare):
+    assert snap_compare(
+        SNAPSHOT_APPS_DIR / "markdown_theme_switcher.py", press=["l", "t", "wait:100"]
+    )
+
+
 def test_checkbox_example(snap_compare):
     assert snap_compare(WIDGET_EXAMPLES_DIR / "checkbox.py")
 
@@ -263,6 +279,10 @@ def test_tabbed_content_with_modified_tabs(snap_compare):
     assert snap_compare(SNAPSHOT_APPS_DIR / "modified_tabs.py")
 
 
+def test_tabbed_content_styling_not_leaking(snap_compare):
+    assert snap_compare(SNAPSHOT_APPS_DIR / "tabbed_content_style_leak_test.py")
+
+
 def test_option_list_strings(snap_compare):
     assert snap_compare(WIDGET_EXAMPLES_DIR / "option_list_strings.py")
 
@@ -297,6 +317,10 @@ def test_option_list_replace_prompt_from_two_lines_to_three_lines(snap_compare):
     )
 
 
+def test_option_list_scrolling_in_long_list(snap_compare):
+    assert snap_compare(SNAPSHOT_APPS_DIR / "option_list_long.py", press=["up"])
+
+
 def test_progress_bar_indeterminate(snap_compare):
     assert snap_compare(WIDGET_EXAMPLES_DIR / "progress_bar_isolated_.py", press=["f"])
 
@@ -351,6 +375,12 @@ def test_select_expanded(snap_compare):
     )
 
 
+def test_select_from_values_expanded(snap_compare):
+    assert snap_compare(
+        WIDGET_EXAMPLES_DIR / "select_from_values_widget.py", press=["tab", "enter"]
+    )
+
+
 def test_select_expanded_changed(snap_compare):
     assert snap_compare(
         WIDGET_EXAMPLES_DIR / "select_widget.py",
@@ -398,6 +428,19 @@ def test_collapsible_custom_symbol(snap_compare):
     assert snap_compare(WIDGET_EXAMPLES_DIR / "collapsible_custom_symbol.py")
 
 
+def test_directory_tree_reloading(snap_compare, tmp_path):
+    async def run_before(pilot):
+        await pilot.app.setup(tmp_path)
+        await pilot.press(
+            "e", "e", "down", "down", "down", "down", "e", "down", "d", "r"
+        )
+
+    assert snap_compare(
+        SNAPSHOT_APPS_DIR / "directory_tree_reload.py",
+        run_before=run_before,
+    )
+
+
 # --- CSS properties ---
 # We have a canonical example for each CSS property that is shown in their docs.
 # If any of these change, something has likely broken, so snapshot each of them.
@@ -473,15 +516,6 @@ def test_demo(snap_compare):
     )
 
 
-# def test_demo_with_keys(snap_compare):
-#     """Test the demo app (python -m textual)"""
-#     assert snap_compare(
-#         Path("../../src/textual/demo.py"),
-#         press=["down", "down", "down", "wait:500"],
-#         terminal_size=(100, 30),
-#     )
-
-
 def test_label_widths(snap_compare):
     """Test renderable widths are calculate correctly."""
     assert snap_compare(SNAPSHOT_APPS_DIR / "label_widths.py")
@@ -532,6 +566,27 @@ def test_richlog_scroll(snap_compare):
     assert snap_compare(SNAPSHOT_APPS_DIR / "richlog_scroll.py")
 
 
+def test_richlog_width(snap_compare):
+    """Check that min_width applies in RichLog and that we can write
+    to the RichLog when it's not visible, and it still renders as expected
+    when made visible again."""
+
+    async def setup(pilot):
+        from rich.text import Text
+
+        rich_log: RichLog = pilot.app.query_one(RichLog)
+        rich_log.write(Text("hello1", style="on red", justify="right"), expand=True)
+        rich_log.visible = False
+        rich_log.write(Text("world2", style="on green", justify="right"), expand=True)
+        rich_log.visible = True
+        rich_log.write(Text("hello3", style="on blue", justify="right"), expand=True)
+        rich_log.display = False
+        rich_log.write(Text("world4", style="on yellow", justify="right"), expand=True)
+        rich_log.display = True
+
+    assert snap_compare(SNAPSHOT_APPS_DIR / "richlog_width.py", run_before=setup)
+
+
 def test_tabs_invalidate(snap_compare):
     assert snap_compare(
         SNAPSHOT_APPS_DIR / "tabs_invalidate.py",
@@ -545,6 +600,18 @@ def test_scrollbar_thumb_height(snap_compare):
     )
 
 
+def test_pilot_resize_terminal(snap_compare):
+    async def run_before(pilot):
+        await pilot.resize_terminal(35, 20)
+        await pilot.resize_terminal(20, 10)
+
+    assert snap_compare(
+        SNAPSHOT_APPS_DIR / "pilot_resize_terminal.py",
+        run_before=run_before,
+        terminal_size=(80, 25),
+    )
+
+
 def test_css_hot_reloading(snap_compare, monkeypatch):
     """Regression test for https://github.com/Textualize/textual/issues/2063."""
 
@@ -600,6 +667,31 @@ def test_datatable_hot_reloading(snap_compare, monkeypatch):
     )
 
 
+def test_markdown_component_classes_reloading(snap_compare, monkeypatch):
+    """Tests all markdown component classes reload correctly.
+
+    See https://github.com/Textualize/textual/issues/3464."""
+
+    monkeypatch.setenv(
+        "TEXTUAL", "debug"
+    )  # This will make sure we create a file monitor.
+
+    async def run_before(pilot):
+        css_file = pilot.app.CSS_PATH
+        with open(css_file, "w") as f:
+            f.write("/* This file is purposefully empty. */\n")  # Clear all the CSS.
+        await pilot.app._on_css_change()
+
+    assert snap_compare(
+        SNAPSHOT_APPS_DIR / "markdown_component_classes_reloading.py",
+        run_before=run_before,
+    )
+
+
+def test_markdown_space_squashing(snap_compare):
+    assert snap_compare(SNAPSHOT_APPS_DIR / "markdown_whitespace.py")
+
+
 def test_layer_fix(snap_compare):
     # Check https://github.com/Textualize/textual/issues/1358
     assert snap_compare(SNAPSHOT_APPS_DIR / "layer_fix.py", press=["d"])
@@ -719,6 +811,16 @@ def test_command_palette(snap_compare) -> None:
     assert snap_compare(SNAPSHOT_APPS_DIR / "command_palette.py", run_before=run_before)
 
 
+def test_command_palette_discovery(snap_compare) -> None:
+    async def run_before(pilot) -> None:
+        pilot.app.screen.query_one(Input).cursor_blink = False
+        await pilot.app.screen.workers.wait_for_complete()
+
+    assert snap_compare(
+        SNAPSHOT_APPS_DIR / "command_palette_discovery.py", run_before=run_before
+    )
+
+
 # --- textual-dev library preview tests ---
 
 
@@ -789,6 +891,7 @@ def test_nested_fr(snap_compare) -> None:
     assert snap_compare(SNAPSHOT_APPS_DIR / "nested_fr.py")
 
 
+@pytest.mark.syntax
 @pytest.mark.parametrize("language", BUILTIN_LANGUAGES)
 def test_text_area_language_rendering(language, snap_compare):
     # This test will fail if we're missing a snapshot test for a valid
@@ -805,7 +908,7 @@ def test_text_area_language_rendering(language, snap_compare):
     assert snap_compare(
         SNAPSHOT_APPS_DIR / "text_area.py",
         run_before=setup_language,
-        terminal_size=(80, snippet.count("\n") + 2),
+        terminal_size=(80, snippet.count("\n") + 4),
     )
 
 
@@ -836,10 +939,25 @@ I am the final line."""
     assert snap_compare(
         SNAPSHOT_APPS_DIR / "text_area.py",
         run_before=setup_selection,
-        terminal_size=(30, text.count("\n") + 1),
+        terminal_size=(30, text.count("\n") + 4),
     )
 
 
+def test_text_area_read_only_cursor_rendering(snap_compare):
+    def setup_selection(pilot):
+        text_area = pilot.app.query_one(TextArea)
+        text_area.theme = "css"
+        text_area.text = "Hello, world!"
+        text_area.read_only = True
+
+    assert snap_compare(
+        SNAPSHOT_APPS_DIR / "text_area.py",
+        run_before=setup_selection,
+        terminal_size=(30, 5),
+    )
+
+
+@pytest.mark.syntax
 @pytest.mark.parametrize(
     "theme_name", [theme.name for theme in TextAreaTheme.builtin_themes()]
 )
@@ -865,7 +983,20 @@ def hello(name):
     assert snap_compare(
         SNAPSHOT_APPS_DIR / "text_area.py",
         run_before=setup_theme,
-        terminal_size=(48, text.count("\n") + 2),
+        terminal_size=(48, text.count("\n") + 4),
+    )
+
+
+def test_text_area_alternate_screen(snap_compare):
+    assert snap_compare(
+        SNAPSHOT_APPS_DIR / "text_area_alternate_screen.py", terminal_size=(48, 10)
+    )
+
+
+@pytest.mark.syntax
+def test_text_area_wrapping_and_folding(snap_compare):
+    assert snap_compare(
+        SNAPSHOT_APPS_DIR / "text_area_wrapping.py", terminal_size=(20, 26)
     )
 
 
@@ -892,9 +1023,11 @@ def test_unscoped_css(snap_compare) -> None:
 def test_big_buttons(snap_compare) -> None:
     assert snap_compare(SNAPSHOT_APPS_DIR / "big_button.py")
 
+
 def test_keyline(snap_compare) -> None:
     assert snap_compare(SNAPSHOT_APPS_DIR / "keyline.py")
 
+
 def test_button_outline(snap_compare):
     """Outline style rendered incorrectly when applied to a `Button` widget.
 
@@ -928,3 +1061,225 @@ def test_vertical_max_height(snap_compare):
     """Test vertical max height takes border in to account."""
     assert snap_compare(SNAPSHOT_APPS_DIR / "vertical_max_height.py")
 
+
+def test_max_height_100(snap_compare):
+    """Test vertical max height takes border in to account."""
+    assert snap_compare(SNAPSHOT_APPS_DIR / "max_height_100.py")
+
+
+def test_loading_indicator(snap_compare):
+    """Test loading indicator."""
+    # https://github.com/Textualize/textual/pull/3816
+    assert snap_compare(SNAPSHOT_APPS_DIR / "loading.py", press=["space"])
+
+
+def test_loading_indicator_disables_widget(snap_compare):
+    """Test loading indicator disabled widget."""
+    # https://github.com/Textualize/textual/pull/3816
+    assert snap_compare(
+        SNAPSHOT_APPS_DIR / "loading.py", press=["space", "down", "down", "space"]
+    )
+
+
+def test_mount_style_fix(snap_compare):
+    """Regression test for broken style update on mount."""
+    # https://github.com/Textualize/textual/issues/3858
+    assert snap_compare(SNAPSHOT_APPS_DIR / "mount_style_fix.py")
+
+
+def test_zero_scrollbar_size(snap_compare):
+    """Regression test for missing content with 0 sized scrollbars"""
+    # https://github.com/Textualize/textual/issues/3886
+    assert snap_compare(SNAPSHOT_APPS_DIR / "zero_scrollbar_size.py")
+
+
+def test_tree_clearing_and_expansion(snap_compare):
+    """Test the Tree.root.is_expanded state after a Tree.clear"""
+    # https://github.com/Textualize/textual/issues/3557
+    assert snap_compare(SNAPSHOT_APPS_DIR / "tree_clearing.py")
+
+
+def test_nested_specificity(snap_compare):
+    """Test specificity of nested rules is working."""
+    # https://github.com/Textualize/textual/issues/3961
+    assert snap_compare(SNAPSHOT_APPS_DIR / "nested_specificity.py")
+
+
+def test_tab_rename(snap_compare):
+    """Test setting a new label for a tab amongst a TabbedContent."""
+    assert snap_compare(SNAPSHOT_APPS_DIR / "tab_rename.py")
+
+
+def test_input_percentage_width(snap_compare):
+    """Check percentage widths work correctly."""
+    # https://github.com/Textualize/textual/issues/3721
+    assert snap_compare(SNAPSHOT_APPS_DIR / "input_percentage_width.py")
+
+
+def test_recompose(snap_compare):
+    """Check recompose works."""
+    # https://github.com/Textualize/textual/pull/4206
+    assert snap_compare(SNAPSHOT_APPS_DIR / "recompose.py")
+
+
+@pytest.mark.parametrize("dark", [True, False])
+def test_ansi_color_mapping(snap_compare, dark):
+    """Test how ANSI colors in Rich renderables are mapped to hex colors."""
+
+    def setup(pilot):
+        pilot.app.dark = dark
+
+    assert snap_compare(SNAPSHOT_APPS_DIR / "ansi_mapping.py", run_before=setup)
+
+
+def test_pretty_grid_gutter_interaction(snap_compare):
+    """Regression test for https://github.com/Textualize/textual/pull/4219."""
+    assert snap_compare(
+        SNAPSHOT_APPS_DIR / "pretty_grid_gutter_interaction.py", terminal_size=(81, 7)
+    )
+
+
+def test_sort_children(snap_compare):
+    """Test sort_children method."""
+    assert snap_compare(SNAPSHOT_APPS_DIR / "sort_children.py", terminal_size=(80, 25))
+
+
+def test_app_blur(snap_compare):
+    """Test Styling after receiving an AppBlur message."""
+
+    async def run_before(pilot) -> None:
+        await pilot.pause()  # Allow the AppBlur message to get processed.
+
+    assert snap_compare(SNAPSHOT_APPS_DIR / "app_blur.py", run_before=run_before)
+
+
+def test_placeholder_disabled(snap_compare):
+    """Test placeholder with diabled set to True."""
+    assert snap_compare(SNAPSHOT_APPS_DIR / "placeholder_disabled.py")
+
+
+def test_listview_index(snap_compare):
+    """Tests that ListView scrolls correctly after updating its index."""
+    assert snap_compare(SNAPSHOT_APPS_DIR / "listview_index.py")
+
+
+def test_button_widths(snap_compare):
+    """Test that button widths expand auto containers as expected."""
+    # https://github.com/Textualize/textual/issues/4024
+    assert snap_compare(SNAPSHOT_APPS_DIR / "button_widths.py")
+
+
+def test_welcome(snap_compare):
+    assert snap_compare(SNAPSHOT_APPS_DIR / "welcome_widget.py")
+
+
+# --- Example apps ---
+# We skip the code browser because the length of the scrollbar in the tree depends on
+# the number of files and folders we have locally and that typically differs from the
+# pristine setting in which CI runs.
+
+
+def test_example_calculator(snap_compare):
+    """Test the calculator example."""
+    assert snap_compare(EXAMPLES_DIR / "calculator.py")
+
+
+def test_example_color_command(snap_compare):
+    """Test the color_command example."""
+    assert snap_compare(
+        EXAMPLES_DIR / "color_command.py",
+        press=["ctrl+backslash", "r", "e", "d", "down", "down", "enter"],
+    )
+
+
+def test_example_dictionary(snap_compare):
+    """Test the dictionary example (basic layout test)."""
+
+    async def run_before(pilot):
+        pilot.app.query(Input).first().cursor_blink = False
+
+    assert snap_compare(EXAMPLES_DIR / "dictionary.py", run_before=run_before)
+
+
+def test_example_five_by_five(snap_compare):
+    """Test the five_by_five example."""
+    assert snap_compare(EXAMPLES_DIR / "five_by_five.py")
+
+
+def test_example_json_tree(snap_compare):
+    """Test the json_tree example."""
+    assert snap_compare(
+        EXAMPLES_DIR / "json_tree.py",
+        press=["a", "down", "enter", "down", "down", "enter"],
+    )
+
+
+def test_example_markdown(snap_compare):
+    """Test the markdown example."""
+    assert snap_compare(EXAMPLES_DIR / "markdown.py")
+
+
+def test_example_merlin(snap_compare):
+    """Test the merlin example."""
+    on_switches = {2, 3, 5, 8}
+
+    async def run_before(pilot):
+        pilot.app.query_one("Timer").running = False  # This will freeze the clock.
+        for switch in pilot.app.query("LabelSwitch"):
+            switch.query_one("Switch").value = switch.switch_no in on_switches
+        await pilot.pause()
+
+    assert snap_compare(EXAMPLES_DIR / "merlin.py", run_before=run_before)
+
+
+def test_example_pride(snap_compare):
+    """Test the pride example."""
+    assert snap_compare(EXAMPLES_DIR / "pride.py")
+
+
+def test_button_with_console_markup(snap_compare):
+    """Regression test for https://github.com/Textualize/textual/issues/4328"""
+    assert snap_compare(SNAPSHOT_APPS_DIR / "button_markup.py")
+
+
+def test_width_100(snap_compare):
+    """Regression test for https://github.com/Textualize/textual/issues/4360"""
+    assert snap_compare(SNAPSHOT_APPS_DIR / "width_100.py")
+
+
+def test_button_with_multiline_label(snap_compare):
+    assert snap_compare(SNAPSHOT_APPS_DIR / "button_multiline_label.py")
+
+
+def test_margin_multiple(snap_compare):
+    assert snap_compare(SNAPSHOT_APPS_DIR / "margin_multiple.py")
+
+
+def test_dynamic_bindings(snap_compare):
+    assert snap_compare(
+        SNAPSHOT_APPS_DIR / "dynamic_bindings.py", press=["a", "b", "c"]
+    )
+
+
+def test_grid_gutter(snap_compare):
+    # https://github.com/Textualize/textual/issues/4522
+    assert snap_compare(SNAPSHOT_APPS_DIR / "grid_gutter.py")
+
+
+def test_multi_keys(snap_compare):
+    # https://github.com/Textualize/textual/issues/4542
+    assert snap_compare(SNAPSHOT_APPS_DIR / "multi_keys.py")
+
+
+def test_data_table_in_tabs(snap_compare):
+    assert snap_compare(SNAPSHOT_APPS_DIR / "data_table_tabs.py")
+
+
+def test_auto_tab_active(snap_compare):
+    # https://github.com/Textualize/textual/issues/4593
+    assert snap_compare(SNAPSHOT_APPS_DIR / "auto_tab_active.py", press=["space"])
+
+
+def test_hatch(snap_compare):
+    """Test hatch styles."""
+    assert snap_compare(SNAPSHOT_APPS_DIR / "hatch.py")
