@@ -447,3 +447,56 @@ def test_split_horizontal_negative():
         Region(10, 5, 22, 14),
         Region(10, 19, 22, 1),
     )
+
+
+def test_translate_inside():
+    # Needs to be moved up
+    assert Region(10, 20, 10, 20).translate_inside(Region(0, 0, 30, 25)) == Region(
+        10, 5, 10, 20
+    )
+
+    # Already inside
+    assert Region(10, 10, 20, 5).translate_inside(Region(0, 0, 100, 100)) == Region(
+        10, 10, 20, 5
+    )
+
+
+def test_inflect():
+    # Default inflect positive
+    assert Region(10, 10, 30, 20).inflect(margin=Spacing(2, 2, 2, 2)) == Region(
+        44, 34, 30, 20
+    )
+
+    # Inflect y axis negative
+    assert Region(10, 10, 30, 20).inflect(
+        y_axis=-1, margin=Spacing(2, 2, 2, 2)
+    ) == Region(44, -14, 30, 20)
+
+    # Inflect y axis negative
+    assert Region(10, 10, 30, 20).inflect(
+        x_axis=-1, margin=Spacing(2, 2, 2, 2)
+    ) == Region(-24, 34, 30, 20)
+
+
+def test_size_with_height():
+    """Test Size.with_height"""
+    assert Size(1, 2).with_height(10) == Size(1, 10)
+
+
+def test_size_with_width():
+    """Test Size.with_width"""
+    assert Size(1, 2).with_width(10) == Size(10, 2)
+
+
+def test_offset_clamp():
+    assert Offset(1, 2).clamp(3, 3) == Offset(1, 2)
+    assert Offset(3, 2).clamp(3, 3) == Offset(2, 2)
+    assert Offset(-3, 2).clamp(3, 3) == Offset(0, 2)
+    assert Offset(5, 4).clamp(3, 3) == Offset(2, 2)
+
+
+def test_size_clamp_offset():
+    assert Size(3, 3).clamp_offset(Offset(1, 2)) == Offset(1, 2)
+    assert Size(3, 3).clamp_offset(Offset(3, 2)) == Offset(2, 2)
+    assert Size(3, 3).clamp_offset(Offset(-3, 2)) == Offset(0, 2)
+    assert Size(3, 3).clamp_offset(Offset(5, 4)) == Offset(2, 2)
