@@ -11,6 +11,7 @@ from rich.style import Style
 from rich.text import Span
 
 import textual.widgets._markdown as MD
+from textual import on
 from textual.app import App, ComposeResult
 from textual.widget import Widget
 from textual.widgets import Markdown
@@ -132,7 +133,7 @@ async def test_load_non_existing_file() -> None:
     [
         ("hello-world", False),
         ("hello-there", True),
-    ]
+    ],
 )
 async def test_goto_anchor(anchor: str, found: bool) -> None:
     """Going to anchors should return a boolean: whether the anchor was found."""
@@ -140,3 +141,26 @@ async def test_goto_anchor(anchor: str, found: bool) -> None:
     async with MarkdownApp(document).run_test() as pilot:
         markdown = pilot.app.query_one(Markdown)
         assert markdown.goto_anchor(anchor) is found
+
+
+async def test_update_of_document_posts_table_of_content_update_message() -> None:
+    """Updating the document should post a TableOfContentsUpdated message."""
+
+    messages: list[str] = []
+
+    class TableOfContentApp(App[None]):
+        def compose(self) -> ComposeResult:
+            yield Markdown("# One\n\n#Two\n")
+
+        @on(Markdown.TableOfContentsUpdated)
+        def log_table_of_content_update(
+            self, event: Markdown.TableOfContentsUpdated
+        ) -> None:
+            nonlocal messages
+            messages.append(event.__class__.__name__)
+
+    async with TableOfContentApp().run_test() as pilot:
+        assert messages == ["TableOfContentsUpdated"]
+        await pilot.app.query_one(Markdown).update("")
+        await pilot.pause()
+        assert messages == ["TableOfContentsUpdated", "TableOfContentsUpdated"]
